@@ -1,5 +1,3 @@
-from __future__ import print_function
-
 import colorsys
 import datetime
 import logging
@@ -9,7 +7,8 @@ import threading
 import time
 from enum import Enum
 
-from .const import (
+from .const import (  # imported for back compat, remove once Home Assistant no longer uses
+    COLOR_MODE_ADDRESSABLE,
     COLOR_MODE_CCT,
     COLOR_MODE_DIM,
     COLOR_MODE_RGB,
@@ -19,6 +18,9 @@ from .const import (
     COLOR_MODES_RGB_CCT,
     COLOR_MODES_RGB_W,
     DEFAULT_MODE,
+    CHANNEL_STATES,
+    MAX_TEMP,
+    MIN_TEMP,
     MODE_COLOR,
     MODE_CUSTOM,
     MODE_MUSIC,
@@ -26,10 +28,22 @@ from .const import (
     MODE_SWITCH,
     MODE_WW,
     MODEL_NUM_SWITCH,
+    STATE_BLUE,
+    STATE_CHANGE_LATENCY,
+    STATE_COOL_WHITE,
+    STATE_GREEN,
+    STATE_RED,
+    STATE_WARM_WHITE,
     STATIC_MODES,
+    WRITE_ALL_COLORS,
+    WRITE_ALL_WHITES,
+    LevelWriteMode,
 )
 from .models_db import (
     BASE_MODE_MAP,
+    CHANNEL_REMAP,
+    MODEL_DESCRIPTIONS,
+    UNKNOWN_MODEL,
     MODEL_MAP,
     RGBW_PROTOCOL_MODELS,
     USE_9BYTE_PROTOCOL_MODELS,
@@ -39,19 +53,13 @@ from .protocol import (
     PROTOCOL_LEDENET_8BYTE,
     PROTOCOL_LEDENET_9BYTE,
     PROTOCOL_LEDENET_ORIGINAL,
-    LevelWriteMode,
     ProtocolLEDENET8Byte,
     ProtocolLEDENET9Byte,
     ProtocolLEDENETOriginal,
 )
 from .sock import _socket_retry
 from .timer import BuiltInTimer, LedTimer
-from .utils import utils
-
-STATE_CHANGE_LATENCY = 1
-MIN_TEMP = 2700
-MAX_TEMP = 6500
-
+from .utils import color_temp_to_white_levels, utils, white_levels_to_color_temp
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -61,32 +69,45 @@ class DeviceType(Enum):
     Switch = 1
 
 
-class WifiLedBulb:
+class LEDENETDevice:
+    """An LEDENET Device."""
+
     def __init__(self, ipaddr, port=5577, timeout=5):
+        """Init the LEDENEt Device."""
         self.ipaddr = ipaddr
         self.port = port
         self.timeout = timeout
         self.raw_state = None
+        self.available = None
+        self._ignore_next_power_state_update = False
 
         self._protocol = None
-
-        self.available = None
-        self._is_on = False
         self._mode = None
-        self._socket = None
-
         self._transition_complete_time = 0
 
-        self._lock = threading.Lock()
-
-        self.connect(retry=2)
-        self.update_state()
-
     @property
     def model_num(self):
         """Return the model number."""
         return self.raw_state.model_num if self.raw_state else None
 
+    @property
+    def model(self):
+        """Return the human readable model description."""
+        model_num = self.model_num
+        description = MODEL_DESCRIPTIONS.get(model_num) or UNKNOWN_MODEL
+        return f"{description} (0x{model_num:02X})"
+
+    @property
+    def version_num(self):
+        """Return the version number."""
+        raw_state = self.raw_state
+        return raw_state.version_number if hasattr(raw_state, "version_number") else 1
+
+    @property
+    def preset_pattern_num(self):
+        """Return the preset pattern number."""
+        return self.raw_state.preset_pattern
+
     @property
     def rgbwprotocol(self):
         """Devices that don't require a separate rgb/w bit."""
@@ -105,6 +126,21 @@ class WifiLedBulb:
             DeviceType.Switch if self.model_num == MODEL_NUM_SWITCH else DeviceType.Bulb
         )
 
+    @property
+    def color_temp(self):
+        """Return the current color temp in kelvin."""
+        return (self.getWhiteTemperature())[0]
+
+    @property
+    def min_temp(self):
+        """Returns the minimum color temp in kelvin."""
+        return MIN_TEMP
+
+    @property
+    def max_temp(self):
+        """Returns the maximum color temp in kelvin."""
+        return MAX_TEMP
+
     @property
     def _rgbwwprotocol(self):
         """Device that uses the 9-byte protocol."""
@@ -119,9 +155,32 @@ class WifiLedBulb:
         """Any white channel is active."""
         return bool(self.raw_state.warm_white or self.raw_state.cool_white)
 
+    @property
+    def color_active(self):
+        """Any color channel is active."""
+        raw_state = self.raw_state
+        return bool(raw_state.red or raw_state.green or raw_state.blue)
+
+    @property
+    def multi_color_mode(self):
+        """The device supports multiple color modes."""
+        return len(self.color_modes) > 1
+
     @property
     def color_modes(self):
         """The available color modes."""
+        color_modes = self._internal_color_modes
+        # We support CCT mode if the device supports RGBWW
+        # but we do not add it to internal color modes as
+        # we need to distingush between devices that are RGB/CCT
+        # and ones that are RGB&CCT
+        if COLOR_MODE_RGBWW in color_modes and COLOR_MODE_CCT not in color_modes:
+            return {COLOR_MODE_CCT, *color_modes}
+        return color_modes
+
+    @property
+    def _internal_color_modes(self):
+        """The internal available color modes."""
         model_db_entry = MODEL_MAP.get(self.model_num)
         if not model_db_entry:
             # Default mode is RGB
@@ -133,7 +192,10 @@ class WifiLedBulb:
     @property
     def color_mode(self):
         """The current color mode."""
-        color_modes = self.color_modes
+        color_modes = self._internal_color_modes
+        if COLOR_MODE_RGBWW in color_modes:
+            # We support CCT mode if the device supports RGBWW
+            return COLOR_MODE_RGBWW if self.color_active else COLOR_MODE_CCT
         if (
             color_modes == COLOR_MODES_RGB_CCT
         ):  # RGB/CCT split, only one active at a time
@@ -153,7 +215,7 @@ class WifiLedBulb:
 
     @property
     def is_on(self):
-        return self._is_on
+        return self.raw_state.power_state == self._protocol.on_byte
 
     @property
     def mode(self):
@@ -200,29 +262,6 @@ class WifiLedBulb:
         # Default color mode (RGB)
         return int(v_255)
 
-    def _connect_if_disconnected(self):
-        """Connect only if not already connected."""
-        if self._socket is None:
-            self.connect()
-
-    @_socket_retry(attempts=0)
-    def connect(self):
-        self.close()
-        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        self._socket.settimeout(self.timeout)
-        _LOGGER.debug("%s: connect", self.ipaddr)
-        self._socket.connect((self.ipaddr, self.port))
-
-    def close(self):
-        if self._socket is None:
-            return
-        try:
-            self._socket.close()
-        except socket.error:
-            pass
-        finally:
-            self._socket = None
-
     def _determineMode(self):
         pattern_code = self.raw_state.preset_pattern
         if self.device_type == DeviceType.Switch:
@@ -239,65 +278,9 @@ class WifiLedBulb:
             return MODE_PRESET
         elif BuiltInTimer.valid(pattern_code):
             return BuiltInTimer.valtostr(pattern_code)
-        return "unknown"
-
-    def _determine_protocol(self):
-        # determine the type of protocol based of first 2 bytes.
-        read_bytes = 2
-
-        protocol = ProtocolLEDENET8Byte()
-        with self._lock:
-            self._connect_if_disconnected()
-            self._send_msg(protocol.construct_state_query())
-            rx = self._read_msg(read_bytes)
-            # if any response is recieved, use the default protocol
-            if len(rx) == read_bytes:
-                # Devices that use an 9-byte protocol
-                if self._uses_9byte_protocol(rx[1]):
-                    self._protocol = ProtocolLEDENET9Byte()
-                else:
-                    self._protocol = protocol
-                return rx + self._read_msg(
-                    self._protocol.state_response_length - read_bytes
-                )
-
-        # We just sent a garage query which the old procotol
-        # cannot process, recycle the connection
-        protocol = ProtocolLEDENETOriginal()
-        self.connect()
-        with self._lock:
-            # if no response from default received, next try the original protocol
-            self._send_msg(protocol.construct_state_query())
-            rx = self._read_msg(read_bytes)
-            if len(rx) == read_bytes and rx[1] == 0x01:
-                self._protocol = protocol
-                return rx + self._read_msg(
-                    self._protocol.state_response_length - read_bytes
-                )
-
-        raise Exception("Cannot determine protocol")
-
-    @_socket_retry(attempts=2)
-    def query_state(self, led_type=None):
-        if led_type:
-            self.setProtocol(led_type)
-        elif not self._protocol:
-            return self._determine_protocol()
-
-        with self._lock:
-            self.connect()
-            self._send_msg(self._protocol.construct_state_query())
-            return self._read_msg(self._protocol.state_response_length)
-
-    def update_state(self, retry=2):
-        rx = self.query_state(retry=retry)
-        if rx and self.process_state_response(rx):
-            self.available = True
-            return
-        self.set_unavailable()
+        return None
 
     def set_unavailable(self):
-        self._is_on = False
         self.available = False
 
     def set_available(self):
@@ -320,20 +303,30 @@ class WifiLedBulb:
         _LOGGER.debug("%s: State: %s", self.ipaddr, raw_state)
 
         if raw_state != self.raw_state:
-            _LOGGER.debug("%s: new_state: %s", self.ipaddr, utils.raw_state_to_dec(rx))
+            _LOGGER.debug(
+                "%s: unmapped raw state: %s",
+                self.ipaddr,
+                utils.raw_state_to_dec(raw_state),
+            )
 
         if time.monotonic() < self._transition_complete_time:
-            # Do not update the raw state if a transition is
+            # Do not update the channel states if a transition is
             # in progress as the state will not be correct
             # until the transition is completed since devices
             # "FADE" into the state requested.
-            return True
+            self._replace_raw_state(
+                {
+                    name: value
+                    for name, value in raw_state._asdict().items()
+                    if name not in CHANNEL_STATES
+                }
+            )
+        else:
+            self._set_raw_state(raw_state)
 
-        self.raw_state = raw_state
-        self._set_power_state_from_raw_state()
         mode = self._determineMode()
 
-        if mode == "unknown":
+        if mode is None:
             _LOGGER.debug(
                 "%s: Unable to determine mode from raw state: %s",
                 self.ipaddr,
@@ -344,13 +337,58 @@ class WifiLedBulb:
         self._mode = mode
         return True
 
-    def _set_power_state_from_raw_state(self):
-        """Set the power state from the raw state."""
-        power_state = self.raw_state.power_state
-        if power_state == self._protocol.on_byte:
-            self._is_on = True
-        elif power_state == self._protocol.off_byte:
-            self._is_on = False
+    def process_power_state_response(self, msg):
+        """Process a power state change message."""
+        if self._ignore_next_power_state_update:
+            # These devices frequently push an incorrect power
+            # state right after changing state.
+            self._ignore_next_power_state_update = False
+            return
+
+        if not self._protocol.is_valid_power_state_response(msg):
+            _LOGGER.warning(
+                "%s: Recieved invalid power state response: %s",
+                self.ipaddr,
+                utils.raw_state_to_dec(msg),
+            )
+            return False
+        if self.is_on and msg[2] == self._protocol.on_byte:
+            # This is a bug in the device firmware
+            _LOGGER.debug(
+                "%s: Device unexpectedly pushed power on when already on, setting to off",
+                self.ipaddr,
+            )
+            self._set_power_state(self._protocol.off_byte)
+        elif not self.is_on and msg[2] == self._protocol.off_byte:
+            # This is a bug in the device firmware
+            _LOGGER.debug(
+                "%s: Device unexpectedly pushed power off when already off, setting to on",
+                self.ipaddr,
+            )
+            self._set_power_state(self._protocol.on_byte)
+        else:
+            _LOGGER.debug(
+                "%s: Setting power state to: %s", self.ipaddr, f"0x{msg[2]:02X}"
+            )
+            self._set_power_state(msg[2])
+
+        return True
+
+    def _set_raw_state(self, raw_state):
+        """Set the raw state remapping channels as needed."""
+        channel_map = CHANNEL_REMAP.get(raw_state.model_num)
+        if not channel_map:  # Remap channels
+            self.raw_state = raw_state
+            return
+        _LOGGER.debug(
+            "%s: remapped raw state: %s", self.ipaddr, utils.raw_state_to_dec(raw_state)
+        )
+        self.raw_state = raw_state._replace(
+            **{
+                mapped: getattr(raw_state, actual)
+                for mapped, actual in channel_map.items()
+            }
+        )
 
     def __str__(self):
         rx = self.raw_state
@@ -374,24 +412,26 @@ class WifiLedBulb:
                 red = rx.red
                 green = rx.green
                 blue = rx.blue
-                mode_str = "Color: {}".format((red, green, blue))
+                mode_str = f"Color: {(red, green, blue)}"
                 # Should add ability to get CCT from rgbwcapable*
                 if self.rgbwcapable:
-                    mode_str += " White: {}".format(ww_level)
+                    mode_str += f" White: {ww_level}"
                 else:
-                    mode_str += " Brightness: {}".format(self.brightness)
+                    mode_str += f" Brightness: {self.brightness}"
             elif color_mode == COLOR_MODE_DIM:
-                mode_str = "Warm White: {}%".format(utils.byteToPercent(ww_level))
+                mode_str = f"Warm White: {utils.byteToPercent(ww_level)}%"
             elif color_mode == COLOR_MODE_CCT:
                 cct_value = self.getWhiteTemperature()
                 mode_str = "CCT: {}K Brightness: {}%".format(
                     cct_value[0], cct_value[1] / 255
                 )
+            elif color_mode == COLOR_MODE_ADDRESSABLE:
+                mode_str = "Addressable"
         elif mode == MODE_PRESET:
             pat = PresetPattern.valtostr(pattern)
-            mode_str = "Pattern: {} (Speed {}%)".format(pat, speed)
+            mode_str = f"Pattern: {pat} (Speed {speed}%)"
         elif mode == MODE_CUSTOM:
-            mode_str = "Custom pattern (Speed {}%)".format(speed)
+            mode_str = f"Custom pattern (Speed {speed}%)"
         elif BuiltInTimer.valid(pattern):
             mode_str = BuiltInTimer.valtostr(pattern)
         elif mode == MODE_MUSIC:
@@ -399,43 +439,23 @@ class WifiLedBulb:
         elif mode == MODE_SWITCH:
             mode_str = "Switch"
         else:
-            mode_str = "Unknown mode 0x{:x}".format(pattern)
+            mode_str = f"Unknown mode 0x{pattern:x}"
         mode_str += " raw state: "
         mode_str += utils.raw_state_to_dec(rx)
-        return "{} [{}]".format(power_str, mode_str)
+        return f"{power_str} [{mode_str}]"
 
-    @_socket_retry(attempts=2)
-    def _change_state(self, turn_on=True):
-        _LOGGER.debug("%s: Changing state to %s", self.ipaddr, turn_on)
-        with self._lock:
-            self._connect_if_disconnected()
-            self._send_msg(self._protocol.construct_state_change(turn_on))
-            # After changing state, the device replies with
-            expected_response_len = 4
-            # - 0x0F 0x71 [0x23|0x24] [CHECK DIGIT]
-            rx = self._read_msg(expected_response_len)
-            _LOGGER.debug("%s: state response %s", self.ipaddr, rx)
-            if len(rx) == expected_response_len:
-                new_power_state = (
-                    self._protocol.on_byte if turn_on else self._protocol.off_byte
-                )
-                self._replace_raw_state({"power_state": new_power_state})
-                self._set_power_state_from_raw_state()
-            self._set_transition_complete_time()
-            # The device will send back a state change here
-            # but it will likely be stale so we want to recycle
-            # the connetion so we do not have to wait as sometimes
-            # it stalls
-            self.close()
+    def _set_power_state_ignore_next_push(self, new_power_state):
+        """Set the power state in the raw state, and ignore next push update."""
+        self._set_power_state(new_power_state)
+        self._ignore_next_power_state_update = True
+
+    def _set_power_state(self, new_power_state):
+        """Set the power state in the raw state."""
+        self._replace_raw_state({"power_state": new_power_state})
+        self._set_transition_complete_time()
 
     def _replace_raw_state(self, new_state):
-        self.raw_state = self.raw_state._replace(**new_state)
-
-    def turnOn(self, retry=2):
-        self._change_state(retry=retry, turn_on=True)
-
-    def turnOff(self, retry=2):
-        self._change_state(retry=retry, turn_on=False)
+        self._set_raw_state(self.raw_state._replace(**new_state))
 
     def isOn(self):
         return self.is_on
@@ -445,46 +465,24 @@ class WifiLedBulb:
             return 255
         return self.brightness
 
-    def setWarmWhite(self, level, persist=True, retry=2):
-        self.setWarmWhite255(utils.percentToByte(level), persist, retry)
-
-    def setWarmWhite255(self, level, persist=True, retry=2):
-        self.setRgbw(w=level, persist=persist, brightness=None, retry=retry)
-
-    def setColdWhite(self, level, persist=True, retry=2):
-        self.setColdWhite255(utils.percentToByte(level), persist, retry)
-
-    def setColdWhite255(self, level, persist=True, retry=2):
-        self.setRgbw(persist=persist, brightness=None, retry=retry, w2=level)
-
-    def setWhiteTemperature(self, temperature, brightness, persist=True, retry=2):
-        # Assume output temperature of between 2700 and 6500 Kelvin, and scale
-        # the warm and cold LEDs linearly to provide that
-        if not (MIN_TEMP <= temperature <= MAX_TEMP):
-            raise ValueError(
-                f"Temperature of {temperature} is not valid and must be between {MIN_TEMP} and {MAX_TEMP}"
-            )
-        brightness = round(brightness / 255, 2)
-        cold = ((6500 - temperature) / (MAX_TEMP - MIN_TEMP)) * (brightness)
-        warm = (brightness) - cold
-        cold = round(255 * cold)
-        warm = round(255 * warm)
-        self.setRgbw(w=cold, w2=warm, persist=persist, retry=retry)
-
     def getWhiteTemperature(self):
         # Assume input temperature of between 2700 and 6500 Kelvin, and scale
         # the warm and cold LEDs linearly to provide that
-        warm = self.raw_state.warm_white / 255
-        cold = self.raw_state.cool_white / 255
-        brightness = warm + cold
-        temperature = ((cold / brightness) * (6493 - 2703)) + 2703
-        brightness = round(brightness * 255)
-        temperature = round(temperature)
-        return (temperature, brightness)
+        raw_state = self.raw_state
+        temp, brightness = white_levels_to_color_temp(
+            raw_state.warm_white, raw_state.cool_white
+        )
+        return temp, brightness
 
     def getRgbw(self):
+        """Returns red,green,blue,white (usually warm)."""
         if self.color_mode not in COLOR_MODES_RGB:
             return (255, 255, 255, 255)
+        return self.rgbw
+
+    @property
+    def rgbw(self):
+        """Returns red,green,blue,white (usually warm)."""
         return (
             self.raw_state.red,
             self.raw_state.green,
@@ -493,8 +491,14 @@ class WifiLedBulb:
         )
 
     def getRgbww(self):
+        """Returns red,green,blue,warm,cool."""
         if self.color_mode not in COLOR_MODES_RGB:
             return (255, 255, 255, 255, 255)
+        return self.rgbww
+
+    @property
+    def rgbww(self):
+        """Returns red,green,blue,warm,cool."""
         return (
             self.raw_state.red,
             self.raw_state.green,
@@ -503,6 +507,23 @@ class WifiLedBulb:
             self.raw_state.cool_white,
         )
 
+    def getRgbcw(self):
+        """Returns red,green,blue,cool,warm."""
+        if self.color_mode not in COLOR_MODES_RGB:
+            return (255, 255, 255, 255, 255)
+        return self.rgbcw
+
+    @property
+    def rgbcw(self):
+        """Returns red,green,blue,cool,warm."""
+        return (
+            self.raw_state.red,
+            self.raw_state.green,
+            self.raw_state.blue,
+            self.raw_state.cool_white,
+            self.raw_state.warm_white,
+        )
+
     def getCCT(self):
         if self.color_mode != COLOR_MODE_CCT:
             return (255, 255)
@@ -513,22 +534,33 @@ class WifiLedBulb:
         speed = utils.delayToSpeed(delay)
         return speed
 
-    @_socket_retry(attempts=2)
-    def setRgbw(
+    def _generate_levels_change(
         self,
-        r=None,
-        g=None,
-        b=None,
-        w=None,
+        channels,
         persist=True,
         brightness=None,
-        w2=None,
     ):
+        """Generate the levels change request."""
+        channel_map = CHANNEL_REMAP.get(self.model_num)
+        if channel_map:
+            mapped_channels = {
+                channel: channels[channel_map.get(channel, channel)]
+                for channel in channels
+            }
+        else:
+            mapped_channels = channels
+
+        r = mapped_channels[STATE_RED]
+        g = mapped_channels[STATE_GREEN]
+        b = mapped_channels[STATE_BLUE]
+        w = mapped_channels[STATE_WARM_WHITE]
+        w2 = mapped_channels[STATE_COOL_WHITE]
+
         if (r or g or b) and (w or w2) and not self.rgbwcapable:
             print("RGBW command sent to non-RGBW device")
             raise ValueError("RGBW command sent to non-RGBW device")
 
-        if brightness != None:
+        if brightness != None and r is not None and g is not None and b is not None:
             (r, g, b) = self._calculateBrightness((r, g, b), brightness)
 
         r_value = 0 if r is None else int(r)
@@ -554,7 +586,7 @@ class WifiLedBulb:
                 write_mode = LevelWriteMode.WHITES
 
         _LOGGER.debug(
-            "%s: setRgbw using %s: persist=%s r=%s, g=%s b=%s, w=%s w2=%s write_mode=%s",
+            "%s: _generate_levels_change using %s: persist=%s r=%s, g=%s b=%s, w=%s w2=%s write_mode=%s",
             self.ipaddr,
             self.protocol,
             persist,
@@ -565,28 +597,17 @@ class WifiLedBulb:
             w2_value,
             write_mode,
         )
+
         msg = self._protocol.construct_levels_change(
             persist, r_value, g_value, b_value, w_value, w2_value, write_mode
         )
-
-        # send the message
-        with self._lock:
-            self._connect_if_disconnected()
-            self._send_msg(msg)
-            updates = {}
-            if len(self.color_modes) > 1 or write_mode in (
-                LevelWriteMode.ALL,
-                LevelWriteMode.COLORS,
-            ):
-                updates.update({"red": r_value, "green": g_value, "blue": b_value})
-            if len(self.color_modes) > 1 or write_mode in (
-                LevelWriteMode.ALL,
-                LevelWriteMode.WHITES,
-            ):
-                updates.update({"warm_white": w_value, "cool_white": w2_value})
-            if updates:
-                self._replace_raw_state(updates)
-            self._set_transition_complete_time()
+        updates = {}
+        multi_mode = self.multi_color_mode
+        if multi_mode or write_mode in WRITE_ALL_COLORS:
+            updates.update({"red": r_value, "green": g_value, "blue": b_value})
+        if multi_mode or write_mode in WRITE_ALL_WHITES:
+            updates.update({"warm_white": w_value, "cool_white": w2_value})
+        return msg, updates
 
     def _set_transition_complete_time(self):
         """Set the time we expect the transition will be completed.
@@ -610,20 +631,184 @@ class WifiLedBulb:
     def getRgb(self):
         if self.color_mode not in COLOR_MODES_RGB:
             return (255, 255, 255)
-        return (self.raw_state.red, self.raw_state.green, self.raw_state.blue)
+        return self.rgb
 
-    def setRgb(self, r, g, b, persist=True, brightness=None, retry=2):
-        self.setRgbw(r, g, b, persist=persist, brightness=brightness, retry=retry)
+    @property
+    def rgb(self):
+        return (self.raw_state.red, self.raw_state.green, self.raw_state.blue)
 
     def _calculateBrightness(self, rgb, level):
         hsv = colorsys.rgb_to_hsv(*rgb)
         return colorsys.hsv_to_rgb(hsv[0], hsv[1], level)
 
+    def setProtocol(self, protocol):
+        if protocol == PROTOCOL_LEDENET_ORIGINAL:
+            self._protocol = ProtocolLEDENETOriginal()
+        elif protocol == PROTOCOL_LEDENET_8BYTE:
+            self._protocol = ProtocolLEDENET8Byte()
+        elif protocol == PROTOCOL_LEDENET_9BYTE:
+            self._protocol = ProtocolLEDENET9Byte()
+        else:
+            raise ValueError(f"Invalid protocol: {protocol}")
+
+    def _generate_preset_pattern(self, pattern, speed):
+        """Generate the preset pattern protocol bytes."""
+        PresetPattern.valtostr(pattern)
+        if not PresetPattern.valid(pattern):
+            raise ValueError("Pattern must be between 0x25 and 0x38")
+        return self._protocol.construct_preset_pattern(pattern, speed)
+
+    def _generate_custom_patterm(self, rgb_list, speed, transition_type):
+        """Generate the custom pattern protocol bytes."""
+        # truncate if more than 16
+        if len(rgb_list) > 16:
+            _LOGGER.warning(
+                "Too many colors in %s, truncating list to %s", len(rgb_list), 16
+            )
+            del rgb_list[16:]
+        # quit if too few
+        if len(rgb_list) == 0:
+            raise ValueError("setCustomPattern requires at least one color tuples")
+
+        return self._protocol.construct_custom_effect(rgb_list, speed, transition_type)
+
+
+class WifiLedBulb(LEDENETDevice):
+    """A LEDENET Wifi bulb device."""
+
+    def __init__(self, ipaddr, port=5577, timeout=5):
+        """Init and setup the bulb."""
+        super().__init__(ipaddr, port, timeout)
+        self._socket = None
+        self._lock = threading.Lock()
+        self.setup()
+
+    def setup(self):
+        """Setup the connection and fetch initial state."""
+        self.connect(retry=2)
+        self.update_state()
+
+    def _connect_if_disconnected(self):
+        """Connect only if not already connected."""
+        if self._socket is None:
+            self.connect()
+
+    @_socket_retry(attempts=0)
+    def connect(self):
+        self.close()
+        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self._socket.settimeout(self.timeout)
+        _LOGGER.debug("%s: connect", self.ipaddr)
+        self._socket.connect((self.ipaddr, self.port))
+
+    def close(self):
+        if self._socket is None:
+            return
+        try:
+            self._socket.close()
+        except OSError:
+            pass
+        finally:
+            self._socket = None
+
+    def turnOn(self, retry=2):
+        self._change_state(retry=retry, turn_on=True)
+
+    def turnOff(self, retry=2):
+        self._change_state(retry=retry, turn_on=False)
+
+    @_socket_retry(attempts=2)
+    def _change_state(self, turn_on=True):
+        _LOGGER.debug("%s: Changing state to %s", self.ipaddr, turn_on)
+        with self._lock:
+            self._connect_if_disconnected()
+            self._send_msg(self._protocol.construct_state_change(turn_on))
+            # After changing state, the device replies with
+            expected_response_len = 4
+            # - 0x0F 0x71 [0x23|0x24] [CHECK DIGIT]
+            rx = self._read_msg(expected_response_len)
+            _LOGGER.debug("%s: state response %s", self.ipaddr, rx)
+            if len(rx) == expected_response_len:
+                # We cannot use the power state workaround here
+                # since we are not listening for power state changes
+                # like the aio version
+                new_power_state = (
+                    self._protocol.on_byte if turn_on else self._protocol.off_byte
+                )
+                self._set_power_state(new_power_state)
+            # The device will send back a state change here
+            # but it will likely be stale so we want to recycle
+            # the connetion so we do not have to wait as sometimes
+            # it stalls
+            self.close()
+
+    def setWarmWhite(self, level, persist=True, retry=2):
+        self.set_levels(w=utils.percentToByte(level), persist=persist, retry=retry)
+
+    def setWarmWhite255(self, level, persist=True, retry=2):
+        self.set_levels(w=level, persist=persist, retry=retry)
+
+    def setColdWhite(self, level, persist=True, retry=2):
+        self.set_levels(w2=utils.percentToByte(level), persist=persist, retry=retry)
+
+    def setColdWhite255(self, level, persist=True, retry=2):
+        self.set_levels(w2=level, persist=persist, retry=retry)
+
+    def setWhiteTemperature(self, temperature, brightness, persist=True, retry=2):
+        cold, warm = color_temp_to_white_levels(temperature, brightness)
+        self.set_levels(w=warm, w2=cold, persist=persist, retry=retry)
+
+    def setRgb(self, r, g, b, persist=True, brightness=None, retry=2):
+        self.set_levels(r, g, b, persist=persist, brightness=brightness, retry=retry)
+
+    def setRgbw(
+        self,
+        r=None,
+        g=None,
+        b=None,
+        w=None,
+        persist=True,
+        brightness=None,
+        w2=None,
+        retry=2,
+    ):
+        return self.set_levels(r, g, b, w, w2, persist, brightness, retry=retry)
+
+    @_socket_retry(attempts=2)
+    def set_levels(
+        self,
+        r=None,
+        g=None,
+        b=None,
+        w=None,
+        w2=None,
+        persist=True,
+        brightness=None,
+    ):
+        msg, updates = self._generate_levels_change(
+            {
+                STATE_RED: r,
+                STATE_GREEN: g,
+                STATE_BLUE: b,
+                STATE_WARM_WHITE: w,
+                STATE_COOL_WHITE: w2,
+            },
+            persist,
+            brightness,
+        )
+        # send the message
+        with self._lock:
+            self._connect_if_disconnected()
+            self._send_msg(msg)
+            if updates:
+                self._replace_raw_state(updates)
+            self._set_transition_complete_time()
+
     def _send_msg(self, bytes):
         _LOGGER.debug(
             "%s => %s (%d)",
             self.ipaddr,
-            " ".join("0x{:02X}".format(x) for x in bytes),
+            " ".join(f"0x{x:02X}" for x in bytes),
             len(bytes),
         )
         self._socket.send(bytes)
@@ -648,14 +833,14 @@ class WifiLedBulb:
                 _LOGGER.debug(
                     "%s <= %s (%d)",
                     self.ipaddr,
-                    " ".join("0x{:02X}".format(x) for x in chunk),
+                    " ".join(f"0x{x:02X}" for x in chunk),
                     len(chunk),
                 )
                 if chunk:
                     begin = time.monotonic()
                 remaining -= len(chunk)
                 rx.extend(chunk)
-            except socket.error as ex:
+            except OSError as ex:
                 _LOGGER.debug("%s: socket error: %s", self.ipaddr, ex)
                 pass
             finally:
@@ -702,33 +887,38 @@ class WifiLedBulb:
             # cycle the connection
             self.close()
 
-    def setProtocol(self, protocol):
-        if protocol == PROTOCOL_LEDENET_ORIGINAL:
-            self._protocol = ProtocolLEDENETOriginal()
-        elif protocol == PROTOCOL_LEDENET_8BYTE:
-            self._protocol = ProtocolLEDENET8Byte()
-        elif protocol == PROTOCOL_LEDENET_9BYTE:
-            self._protocol = ProtocolLEDENET9Byte()
-        else:
-            raise ValueError(f"Invalid protocol: {protocol}")
+    def _determine_protocol(self):
+        # determine the type of protocol based of first 2 bytes.
+        read_bytes = 2
+        for protocol_cls in (ProtocolLEDENET8Byte, ProtocolLEDENETOriginal):
+            protocol = protocol_cls()
+            with self._lock:
+                self._connect_if_disconnected()
+                self._send_msg(protocol.construct_state_query())
+                rx = self._read_msg(read_bytes)
+                # if any response is recieved, use the protocol
+                if len(rx) != read_bytes:
+                    # We just sent a garage query which the old procotol
+                    # cannot process, recycle the connection
+                    self.close()
+                    continue
+                full_msg = rx + self._read_msg(
+                    protocol.state_response_length - read_bytes
+                )
+                if protocol.is_valid_state_response(full_msg):
+                    # Devices that use an 9-byte protocol
+                    if self._uses_9byte_protocol(rx[1]):
+                        self._protocol = ProtocolLEDENET9Byte()
+                    else:
+                        self._protocol = protocol
+                return full_msg
+        raise Exception("Cannot determine protocol")
 
     def setPresetPattern(self, pattern, speed):
-
-        PresetPattern.valtostr(pattern)
-        if not PresetPattern.valid(pattern):
-            # print "Pattern must be between 0x25 and 0x38"
-            raise Exception
-
-        delay = utils.speedToDelay(speed)
-        # print "speed {}, delay 0x{:02x}".format(speed,delay)
-        pattern_set_msg = bytearray([0x61])
-        pattern_set_msg.append(pattern)
-        pattern_set_msg.append(delay)
-        pattern_set_msg.append(0x0F)
-
+        msg = self._generate_preset_pattern(pattern, speed)
         with self._lock:
             self._connect_if_disconnected()
-            self._send_msg(self._protocol.construct_message(pattern_set_msg))
+            self._send_msg(msg)
 
     def getTimers(self):
         msg = bytearray([0x22, 0x2A, 0x2B, 0x0F])
@@ -782,54 +972,34 @@ class WifiLedBulb:
             self._connect_if_disconnected()
             self._send_msg(self._protocol.construct_message(msg))
             # not sure what the resp is, prob some sort of ack?
-            rx = self._read_msg(4)
+            self._read_msg(4)
 
-    def setCustomPattern(self, rgb_list, speed, transition_type):
-        # truncate if more than 16
-        if len(rgb_list) > 16:
-            print("too many colors, truncating list")
-            del rgb_list[16:]
-
-        # quit if too few
-        if len(rgb_list) == 0:
-            print("no colors, aborting")
-            return
-
-        msg = bytearray()
-
-        first_color = True
-        for rgb in rgb_list:
-            if first_color:
-                lead_byte = 0x51
-                first_color = False
-            else:
-                lead_byte = 0
-            r, g, b = rgb
-            msg.extend(bytearray([lead_byte, r, g, b]))
-
-        # pad out empty slots
-        if len(rgb_list) != 16:
-            for i in range(16 - len(rgb_list)):
-                msg.extend(bytearray([0, 1, 2, 3]))
-
-        msg.append(0x00)
-        msg.append(utils.speedToDelay(speed))
-
-        if transition_type == "gradual":
-            msg.append(0x3A)
-        elif transition_type == "jump":
-            msg.append(0x3B)
-        elif transition_type == "strobe":
-            msg.append(0x3C)
-        else:
-            # unknown transition string: using 'gradual'
-            msg.append(0x3A)
-        msg.append(0xFF)
-        msg.append(0x0F)
+    @_socket_retry(attempts=2)
+    def query_state(self, led_type=None):
+        if led_type:
+            self.setProtocol(led_type)
+        elif not self._protocol:
+            return self._determine_protocol()
 
         with self._lock:
-            self._connect_if_disconnected
-            self._send_msg(self._protocol.construct_message(msg))
+            self.connect()
+            self._send_msg(self._protocol.construct_state_query())
+            return self._read_msg(self._protocol.state_response_length)
+
+    def update_state(self, retry=2):
+        rx = self.query_state(retry=retry)
+        if rx and self.process_state_response(rx):
+            self.available = True
+            return
+        self.set_unavailable()
+
+    @_socket_retry(attempts=2)
+    def setCustomPattern(self, rgb_list, speed, transition_type):
+        """Set a custom pattern on the device."""
+        msg = self._generate_custom_patterm(rgb_list, speed, transition_type)
+        with self._lock:
+            self._connect_if_disconnected()
+            self._send_msg(msg)
 
     def refreshState(self):
         return self.update_state()
