@@ -5,6 +5,24 @@ from abc import abstractmethod
 from collections import namedtuple
 from enum import Enum
 
+from .const import (
+    STATE_BLUE,
+    STATE_CHECK_SUM,
+    STATE_COLOR_MODE,
+    STATE_COOL_WHITE,
+    STATE_GREEN,
+    STATE_HEAD,
+    STATE_MODE,
+    STATE_MODEL_NUM,
+    STATE_POWER_STATE,
+    STATE_PRESET_PATTERN,
+    STATE_RED,
+    STATE_SPEED,
+    STATE_VERSION_NUMBER,
+    STATE_WARM_WHITE,
+)
+from .utils import utils
+
 _LOGGER = logging.getLogger(__name__)
 
 # Protocol names
@@ -12,34 +30,36 @@ PROTOCOL_LEDENET_ORIGINAL = "LEDENET_ORIGINAL"
 PROTOCOL_LEDENET_9BYTE = "LEDENET"
 PROTOCOL_LEDENET_8BYTE = "LEDENET_8BYTE"  # Previously was called None
 
+from .const import TRANSITION_GRADUAL, TRANSITION_JUMP, TRANSITION_STROBE
+
+TRANSITION_BYTES = {
+    TRANSITION_JUMP: 0x3B,
+    TRANSITION_STROBE: 0x3C,
+    TRANSITION_GRADUAL: 0x3A,
+}
+
 LEDENET_ORIGINAL_STATE_RESPONSE_LEN = 11
 LEDENET_STATE_RESPONSE_LEN = 14
 
 LEDENET_BASE_STATE = [
-    "head",
-    "model_num",
-    "power_state",
-    "preset_pattern",
-    "mode",
-    "speed",
-    "red",
-    "green",
-    "blue",
-    "warm_white",
+    STATE_HEAD,
+    STATE_MODEL_NUM,
+    STATE_POWER_STATE,
+    STATE_PRESET_PATTERN,
+    STATE_MODE,
+    STATE_SPEED,
+    STATE_RED,
+    STATE_GREEN,
+    STATE_BLUE,
+    STATE_WARM_WHITE,
 ]
 
 
-class LevelWriteMode(Enum):
-    ALL = 0x00
-    COLORS = 0xF0
-    WHITES = 0x0F
-
-
 LEDENETOriginalRawState = namedtuple(
     "LEDENETOriginalRawState",
     [
         *LEDENET_BASE_STATE,
-        "check_sum",
+        STATE_CHECK_SUM,
     ],
 )
 # typical response:
@@ -64,10 +84,10 @@ LEDENETRawState = namedtuple(
     "LEDENETRawState",
     [
         *LEDENET_BASE_STATE,
-        "version_number",
-        "cool_white",
-        "color_mode",
-        "check_sum",
+        STATE_VERSION_NUMBER,
+        STATE_COOL_WHITE,
+        STATE_COLOR_MODE,
+        STATE_CHECK_SUM,
     ],
 )
 # response from a 5-channel LEDENET controller:
@@ -102,6 +122,20 @@ class ProtocolBase:
     def is_valid_state_response(self, raw_state):
         """Check if a state response is valid."""
 
+    def is_checksum_correct(self, msg):
+        """Check a checksum of a message."""
+        expected_sum = sum(msg[0:-1]) & 0xFF
+        if expected_sum != msg[-1]:
+            _LOGGER.warning(
+                "Checksum mismatch: Expected %s, got %s", expected_sum, msg[-1]
+            )
+            return False
+        return True
+
+    @abstractmethod
+    def is_valid_power_state_response(self, msg):
+        """Check if a power state response is valid."""
+
     @property
     def on_byte(self):
         """The on byte."""
@@ -122,6 +156,36 @@ class ProtocolBase:
     ):
         """The bytes to send for a level change request."""
 
+    def construct_preset_pattern(self, pattern, speed):
+        """The bytes to send for a preset pattern."""
+        delay = utils.speedToDelay(speed)
+        return self.construct_message(bytearray([0x61, pattern, delay, 0x0F]))
+
+    def construct_custom_effect(self, rgb_list, speed, transition_type):
+        """The bytes to send for a custom effect."""
+        msg = bytearray()
+        first_color = True
+        for rgb in rgb_list:
+            if first_color:
+                lead_byte = 0x51
+                first_color = False
+            else:
+                lead_byte = 0
+            r, g, b = rgb
+            msg.extend(bytearray([lead_byte, r, g, b]))
+        # pad out empty slots
+        if len(rgb_list) != 16:
+            for i in range(16 - len(rgb_list)):
+                msg.extend(bytearray([0, 1, 2, 3]))
+        msg.append(0x00)
+        msg.append(utils.speedToDelay(speed))
+        msg.append(
+            TRANSITION_BYTES.get(transition_type, TRANSITION_BYTES[TRANSITION_GRADUAL])
+        )  # default to "gradual"
+        msg.append(0xFF)
+        msg.append(0x0F)
+        return self.construct_message(msg)
+
     @property
     @abstractmethod
     def name(self):
@@ -164,6 +228,16 @@ class ProtocolLEDENETOriginal(ProtocolBase):
         """The length of the query response."""
         return LEDENET_ORIGINAL_STATE_RESPONSE_LEN
 
+    def is_valid_power_state_response(self, msg):
+        """Check if a power state response is valid."""
+        # We do not have dumps of the original ledenet
+        # protocol (these devices are no longer made).
+        # If we get them in the future, we can
+        # implement push updates for these devices by
+        # matching how is_valid_power_state_response works
+        # for the newer protocol
+        return False
+
     def is_valid_state_response(self, raw_state):
         """Check if a state response is valid."""
         return len(raw_state) == self.state_response_length and raw_state[1] == 0x01
@@ -215,19 +289,24 @@ class ProtocolLEDENET8Byte(ProtocolBase):
         """The length of the query response."""
         return LEDENET_STATE_RESPONSE_LEN
 
+    def is_valid_power_state_response(self, msg):
+        """Check if a power state response is valid."""
+        if (
+            len(msg) != 4
+            or msg[0] not in (0xF0, 0x00, 0x0F)
+            or msg[1] != 0x71
+            or msg[2] not in (self.on_byte, self.off_byte)
+        ):
+            return False
+        return self.is_checksum_correct(msg)
+
     def is_valid_state_response(self, raw_state):
         """Check if a state response is valid."""
         if len(raw_state) != self.state_response_length:
             return False
         if raw_state[0] != 0x81:
             return False
-        expected_sum = sum(raw_state[0:-1]) & 0xFF
-        if expected_sum != raw_state[-1]:
-            _LOGGER.warning(
-                "Checksum mismatch: Expected %s, got %s", expected_sum, raw_state[-1]
-            )
-            return False
-        return True
+        return self.is_checksum_correct(raw_state)
 
     def construct_state_change(self, turn_on):
         """The bytes to send for a state change request."""
