@@ -16,11 +16,18 @@ def _socket_retry(attempts=2):
                     ret = func(self, *args, **kwargs)
                     self.set_available()
                     return ret
-                except socket.error as ex:
+                except OSError as ex:
                     _LOGGER.debug(
                         "%s: socket error while calling %s: %s", self.ipaddr, func, ex
                     )
-            self.set_unavailable()
+                    if attempts_remaining:
+                        continue
+                    self.set_unavailable()
+                    self.close()
+                    # We need to raise or the bulb will
+                    # always be seen as available in Home Assistant
+                    # when it goes offline
+                    raise
 
         return _retry_wrap
 
